from u3driver.commands import *
from u3driver.altElement import AltElement
from u3driver.by import By
from u3driver.runner import *
# from altunityrunner.alt_unity_port_forwarding import AltUnityAndroidPortForwarding, AltUnityiOSPortForwarding
__version__ = "1.0.0"


import json
import re
import socket
import subprocess
import time
import multiprocessing

from u3driver.altUnityExceptions import *
from deprecated import deprecated
from u3driver.commands import *
from u3driver.altElement import AltElement
BUFFER_SIZE = 1024

class AltrunUnityDriver(object):

    def __init__(self, appium_driver,  platform, TCP_IP='127.0.0.1',TCP_PORT=13000, timeout=60,request_separator=';',request_end='&',device_id="",log_flag=False):
        self.TCP_PORT = TCP_PORT
        self.request_separator=request_separator
        self.request_end=request_end
        self.log_flag=log_flag
        self.appium_driver=None
        self.connect = False
        self.pause = False
        if (appium_driver != None):
            self.appium_driver = appium_driver

        while timeout > 0:
            try:
                self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                self.socket.connect((TCP_IP, TCP_PORT))
                self.socket.settimeout(300)
                # print("Get server Version")
                GetServerVersion(self.socket, self.request_separator, self.request_end).execute()
                self.connect = True
                break
            except Exception as e:
                print(e)
                print('AltUnityServer not running on port ' + str(self.TCP_PORT) +
                      ', retrying (timing out in ' + str(timeout) + ' secs)...')
                timeout -= 5
                time.sleep(5)

        if timeout <= 0:
            raise Exception('Could not connect to AltUnityServer on: '+ TCP_IP +':'+ str(self.TCP_PORT))

    def NeedPause(self):
        while self.pause:
            time.sleep(1)
            print("[Info]udriver is pausing!")
    
    def Pause(self,pause):
        self.pause = pause

    def stop(self):
        self.pause = False
        CloseConnection(self.socket,self.request_separator,self.request_end).execute()

    def find_object(self,by,value,image_url = None):
        self.NeedPause()
        return FindObject(self.socket,self.request_separator,self.request_end,self.appium_driver,by,value,image_url).execute()

    def tap_at_coordinates(self,x,y):
        self.NeedPause()
        return TapAtCoordinates(self.socket,self.request_separator,self.request_end,self.appium_driver,x,y).execute()

    def find_object_and_tap(self,by,value,camera_name='',enabled=True):
        self.NeedPause()
        return FindObjectAndTap(self.socket,self.request_separator,self.request_end,self.appium_driver,by,value,camera_name,enabled).execute()

    def find_object_in_range_where_name_contains(self, name,range_path,camera_name='',enabled=True):
        self.NeedPause()
        return FindObjectInRangeWhereNameContains(self.socket,self.request_separator,self.request_end,self.appium_driver,name,range_path,camera_name,enabled).execute()

    def object_exist(self, by,value):
        self.NeedPause()
        return ObjectExist(self.socket,self.request_separator,self.request_end,self.appium_driver,by,value).execute()

    def find_all_object_where_text_contains(self,text):
        self.NeedPause()
        return FindAllObjectWhereTextContains(self.socket,self.request_separator,self.request_end,self.appium_driver,text).execute()

    def get_screen(self):
        self.NeedPause()
        return GetScreen(self.socket,self.request_separator,self.request_end,self.appium_driver).execute()

    def find_child(self,value):
        self.NeedPause()
        return FindChild(self.socket,self.request_separator,self.request_end,self.appium_driver,value).execute()
    
    def get_object_rect(self,value):
        self.NeedPause()
        return GetObjectRect(self.socket,self.request_separator,self.request_end,self.appium_driver,value).execute()
    
    def find_all_objects(self,value):
        self.NeedPause()
        return FindAllObjects(self.socket,self.request_separator,self.request_end,self.appium_driver,value).execute()

    def find_all_objects_in_range_where_text_contains(self,value,text):
        self.NeedPause()
        return FindAllObjectInRangeWhereTextContains(self.socket,self.request_separator,self.request_end,self.appium_driver,value,text).execute()
    
    #调用格式如下：组件名必须是完整的，而且要带上模块名称
    #udriver.get_value_on_component("//Canvas","Test,Assembly-CSharp","test1")
    #udriver.get_value_on_component("//Canvas","UnityEngine.UI.Text,UnityEngine.UI","text")
    def get_value_on_component(self,path,component_name,value_name):
        self.NeedPause()
        return GetValueOnComponent(self.socket,self.request_separator,self.request_end,self.appium_driver,path,component_name,value_name).execute()

    def debug_mode(self,file_path = None):
        return DebugMode(self.socket,self.request_separator,self.request_end,file_path).execute()

    def drag_object(self,path,x1,y1,x2 = None,y2 = None):
        self.NeedPause()
        return Drag(self.socket,self.request_separator,self.request_end,path,x1,y1,x2,y2).execute()

    
    '''
    用法:
    id = udriver.find_all_objects_where_text_contains("//UICanvas","xxx")[0]["id"]
    udriver.tap_by_id(id)
    '''
    def tap_by_id(self,id):
        # json.dumps({"id",id})
        self.NeedPause()
        return AltElement(CommandReturningAltElements(self.socket,self.request_separator,self.request_end,self.appium_driver),self.appium_driver,'{"name":"","id":"'+ id +'"}').tap()

    def find_all_text(self):
        self.NeedPause()
        return FindAllText(self.socket,self.request_separator,self.request_end,self.appium_driver).execute()

    def get_hierarchy(self):
        self.NeedPause()
        return GetHierarchy(self.socket,self.request_separator,self.request_end,self.appium_driver).execute()

        
    def get_inspector(self, id):
        self.NeedPause()
        return GetInspector(self.socket,self.request_separator,self.request_end,self.appium_driver, id).execute()

    def get_server_version(self):
        return GetServerVersion(self.socket, self.request_separator, self.request_end).execute()


from u3driver.commands.ObjectCommands.get_text import *
from u3driver.commands.ObjectCommands.get_text import *
from u3driver.commands.ObjectCommands.tap import *
from u3driver.commands.ObjectCommands.drag import *


# coding: utf-8
import os, re
from setuptools import setup, find_namespace_packages

with open(os.path.join("u3driver", "__init__.py"), encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)

setup(
    name='u3driver',
    version=version,
    python_requires='>=3.6',
    description='u3driver',
    url='https://github.com/king3soft/u3driver',
    author='king3soft',
    author_email='buutuud@gmail.com',
    license='GPLv3',
    include_package_data=True,
    packages=find_namespace_packages(include=['u3driver.*', "u3driver"]),
    install_requires=''''''.split('\n'),
    zip_safe=False)


from u3driver.commands.base_command import BaseCommand
import time
import json

class DebugMode(BaseCommand):

    def __init__(self, socket,request_separator,request_end,file_path = None):
        super().__init__(socket,request_separator,request_end)
        self.file_path = file_path
    
    def execute(self):
        screen = self.send_data(self.create_command('getScreen'))
        print(screen)
        screen = json.loads(screen)
        screen = {"width":int(screen["width"]), "height":int(screen['height'])}

        response = self.send_data(self.create_command('debugMode', '0'))
        if response == "Open Debug Mode" or response == "Debug Already Opened":
            ##结束行有17行
            end_str = """\texcept Exception as e:\n\t\tprint(f'{e}')\n\t\traise e\n\nif __name__ == '__main__':\n\tparser = argparse.ArgumentParser()\n\tparser.add_argument('-s', help="device serial")\n\tparser.add_argument('-i', help="ip address")\n\tparser.add_argument('-port', help="ip address")\n\targs = parser.parse_args()\n\tdevice_s = args.s\n\tip = args.i\n\tport = int(args.port)\n\tudriver = AltrunUnityDriver(device_s,"", ip, TCP_PORT=port,timeout=60, log_flag=True)\n\tAutoRun(udriver)\n\tudriver.stop()"""
            tap_count = 0
            if self.file_path != None:
                data_str = ''
                data_str += 'from u3driver import AltrunUnityDriver\n'
                data_str += 'from u3driver import By\n'
                data_str += 'import time\n'
                data_str += 'import argparse\n\n'
                data_str += 'def AutoRun(udriver):\n'
                tap_count += 1
                data_str += '\t' * tap_count + 'try:\n'
                tap_count += 1
                
                if self.file_path != None:
                        with open(file=self.file_path, mode='w+', encoding='utf-8') as f:
                            f.write(data_str)
            start_position = None
            end_position = None
            while True:
                data = self.recvall()
                data_str = ''
                if data == "Close Debug Mode":
                    
#                     tap_count -= 1
#                     data_str += '\t' * tap_count + 'except Exception as e:\n'
#                     tap_count += 1
#                     data_str += '\t' * tap_count + 'print(f"{e}")\n'
#                     data_str += '\t' * tap_count + 'raise e\n\n'
#                     tap_count -= 1

#                     data_str += """
# if __name__ == "__main__":
#     parser = argparse.ArgumentParser()
#     parser.add_argument('-s', help="device serial")
#     parser.add_argument('-i', help="ip address")
#     parser.add_argument('-port', help="ip address")
#     args = parser.parse_args()
#     device_s = args.s
#     ip = args.i
#     port = args.port
#     udriver = AltrunUnityDriver(device_s,"", ip, TCP_PORT=port,timeout=60, log_flag=True)
#     AutoRun(udriver)
#     udriver.stop()
# """
#                     if self.file_path != None:
#                         with open(file=self.file_path, mode='a+', encoding='utf-8') as f:
#                             f.write(data_str)
                    return data
                json_data = json.loads(data)
                time = 2
                if "time" in json_data:
                    time = float(json_data['time'])
                if "start position" in json_data:
                    start_position = tuple(eval(json_data['start position']))
                elif "end position" in json_data:
                    end_position = tuple(eval(json_data['end position']))
                    if start_position != None and end_position != None:
                        data_str += '\t' * tap_count + f'time.sleep({time})\n'
                        data_str += '\t' * tap_count + 'udriver.drag_object("'+json_data['name'].replace("/","//")+f'",{start_position[0] / screen["width"]},{start_position[1] / screen["height"]},{end_position[0] / screen["width"]},{end_position[1] / screen["height"]})\n'
                        start_position = None
                        end_position = None
                elif "value" in json_data:
                    data_str += '\t' * tap_count + f'time.sleep({time})\n'
                    data_str += '\t' * tap_count + 'udriver.find_object(By.PATH,"'+json_data['name'].replace("/","//")+'")'
                    data_str += '.set_text("' + json_data['value'] + '")\n'
                elif "name" in json_data:
                    data_str += '\t' * tap_count + f'time.sleep({time})\n'
                    data_str += '\t' * tap_count + 'udriver.find_object(By.PATH,"'+json_data['name'].replace("/","//")+'")'
                    data_str += '.tap()\n'
                if self.file_path != None:
                        all_str = '' 
                        with open(file=self.file_path, mode='r+', encoding='utf-8') as f:
                            all_str = f.read()

                        all_str = all_str.replace(end_str,"")
                        all_str += data_str
                        all_str += end_str

                        with open(file=self.file_path, mode='w+', encoding='utf-8') as f:
                            
                            f.write(all_str)
                
        return response
